/* apccomm_all.c
   Part of "APCComm" 
   Copyright (C) 2000,2001,2006 Ralf Hoffmann
   Contact: ralf@boomerangsworld.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
/* $Id: apccomm_all.c,v 1.9 2006/01/15 22:31:12 ralf Exp $ */

#include "apccomm_all.h"

#if defined AMIGA && defined GUI
#include "apccomm_gui_includes.h"
#include "apccomm_gui.h"
#include "apccomm_am.h"
#endif

const char *aboutStr = "APComm %ld.%ld.%ld\n" \
                 "Copyright (C) 2000-2002  Ralf Hoffmann\n" \
                 "\n" \
                 "This program is free software; you can redistribute it and/or modify\n" \
                 "it under the terms of the GNU General Public License as published by\n" \
                 "the Free Software Foundation; either version 2 of the License, or\n" \
                 "(at your option) any later version.\n" \
                 "\n" \
                 "This program is distributed in the hope that it will be useful,\n" \
                 "but WITHOUT ANY WARRANTY; without even the implied warranty of\n" \
                 "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n" \
                 "GNU General Public License for more details.\n" \
                 "\n" \
                 "You should have received a copy of the GNU General Public License\n" \
                 "along with this program; if not, write to the Free Software\n" \
                 "Foundation, Inc., 59 Temple Place - Suite 330,\n" \
                 "Boston, MA 02111-1307, USA.\n" \
                 "\n" \
                 "Use this at your own risk. A basic checksum code is used but there is\n" \
                 "no warranty for correct transmission. You have been warned!\n" \
                 "I'm not responsible for any error or damage that occurs by using\n" \
                 "this program.";

const char *aboutStr2 = "For more information or new version visit the homepage:\n" \
                  "http://www.boomerangsworld.de/apccomm\n" \
                  "\n" \
                  "E-Mail: Ralf Hoffmann <ralf@boomerangsworld.de>" \
                  "\0" AMIGA_VERSION_STR;

unsigned char *outblock = NULL, *inblock = NULL;
pc_file_t outfile;
pc_file_t infile;

char *basedir = NULL;
int verbose = 0;
int ignore_prot = 0;

enum { OVERWRITE_REQUEST, OVERWRITE_ALL, OVERWRITE_NONE } overwrite_mode = OVERWRITE_REQUEST;

int init_transferblocks( void )
{
  int i;

  outblock=(unsigned char*)malloc(TRANSFERBLOCKSIZE);
  inblock=(unsigned char*)malloc(TRANSFERBLOCKSIZE);
  if(outblock!=NULL)
    for(i=0;i<TRANSFERBLOCKSIZE;i++) outblock[i]=0;
  if(inblock!=NULL)
    for(i=0;i<TRANSFERBLOCKSIZE;i++) inblock[i]=0;
  if((inblock==NULL)||(outblock==NULL)) return 1;
  return 0;
}

int init_outgoing_file(const char*name)
{
  int fp;
  FILE *tfp;
  int size;
  
  outfile.size=0;
  outfile.checksum=0;
  outfile.fp=0;
  
  tfp=fopen(name,"r");
  if(tfp!=NULL) {
    fseek(tfp,0,SEEK_END);
    size=ftell(tfp);
    fseek(tfp,0,SEEK_SET);
    outfile.size=size;
    fp=open(name,O_RDONLY);
    fclose(tfp);
    outfile.fp=fp;
  } else return 1;
  outfile.name=strdup(name);
  return 0;
}

int init_incoming_file(const char*name)
{
  FILE *tfp;
  int fp;
  char *newname, *basename, *tstr=NULL;
#ifndef GUI
  char choose[8];
#endif
  int fileok;
#if defined AMIGA && defined GUI
  int res;
#endif
  
  infile.size=0;
  infile.checksum=0;
  infile.fp=0;

  newname = (char*)malloc( 256 );
  if ( newname == NULL ) {
#if defined AMIGA && defined GUI
    rtEZRequest( "No mem!", "Okay", NULL, (struct TagItem *)rttags );
#else
    printf( "No mem!\n" );
#endif
    return -2;
  }

  if(basedir!=NULL) {
    basename = strdup( basedir );
  } else {
    basename=strdup("");
  }
  
  strcpy(newname,name);
  fileok=0;
  do {
    if(tstr!=NULL) free(tstr);
    tstr=(char*)malloc(strlen(basename)+strlen(newname)+1);
    sprintf(tstr,"%s%s",basename,newname);
    tfp=fopen(tstr,"r");
    if(tfp!=NULL) {
      fclose(tfp);
      if ( overwrite_mode == OVERWRITE_ALL ) {
	fileok = 1;
      } else if ( overwrite_mode == OVERWRITE_NONE ) {
	fileok = -1;
      } else {
#if defined AMIGA && defined GUI
	res = rtEZRequest( "File %s already exists!", "Change Name|Overwrite|Overwrite all|Skip|Skip all|Abort", NULL, (struct TagItem *)rttags, tstr );
	if ( res == 2 ) {
	  fileok = 1;
	} else if ( res == 3 ) {
	  fileok = 1;
	  overwrite_mode = OVERWRITE_ALL;
	} else if ( res == 4 ) {
	  fileok = -1;
	} else if ( res == 5 ) {
	  fileok = -1;
	  overwrite_mode = OVERWRITE_NONE;
	} else if ( res == 0 ) {
	  fileok = -2;
	} else {
	  if ( ! rtGetStringA( newname, 255, "Enter new name:", NULL, (struct TagItem *)rttags ) ) {
	    fileok = -2;
	  }
	}
#else
	printf("File %s already exists!\n\tOverwrite (o), Overwrite all (o!),\n\tSkip (s), Skip all(s!),\n\tchange (c) name or abort (a):",tstr);
	fflush(stdout);
	scanf( "%4s", choose );
	if(choose[0]=='o') {
	  fileok=1;
	  if ( choose[1] == '!' ) overwrite_mode = OVERWRITE_ALL;
	} else if ( choose[0] == 's' ) {
	  fileok = -1;
	  if ( choose[1] == '!' ) overwrite_mode = OVERWRITE_NONE;
	} else if ( choose[0] == 'a' ) {
	  fileok = -2;
	} else {
	  printf("Enter new name:");
	  fflush(stdout);
	  scanf( "%255s", newname );
	}
#endif
      }
    } else fileok=1;
  } while(fileok==0);
  free( basename );
  if ( fileok < 0 ) {
    free( tstr );
    free( newname );
    return fileok;
  }
  fp=open(tstr,O_WRONLY|O_CREAT|O_TRUNC,0644);
  infile.fp=fp;
  if(fp<0) {
    free(tstr);
    free( newname );
    return 1;
  }
  infile.name = newname;
  free(tstr);
  return 0;
}

void close_outgoing_file( void )
{
  if(outfile.fp!=0) close(outfile.fp);
}

void close_incoming_file( void )
{
  if(infile.fp!=0) close(infile.fp);
}

void putint(unsigned char *buffer,int value)
{
  int b3,b2,b1,b0;
  b0=value&0xff;
  value>>=8;
  b1=value&0xff;
  value>>=8;
  b2=value&0xff;
  value>>=8;
  b3=value&0xff;
  *buffer++=(unsigned char)b3;
  *buffer++=(unsigned char)b2;
  *buffer++=(unsigned char)b1;
  *buffer++=(unsigned char)b0;
}

int getint(unsigned char *buffer)
{
  int value;
  value=(int)*buffer++;
  value<<=8;
  value|=(int)*buffer++;
  value<<=8;
  value|=(int)*buffer++;
  value<<=8;
  value|=(int)*buffer++;
  return value;
}

int readtobuffer(unsigned char* buffer,int maxsize,int *return_checksum)
{
  int readed,checksum,i;
  readed=read(outfile.fp,buffer,maxsize);
  checksum=0;
  for(i=0;i<readed;i++) checksum+=(int)buffer[i];
  outfile.checksum+=checksum;
  if(return_checksum!=NULL) *return_checksum=checksum;
  return readed;
}

int writefrombuffer(unsigned char *buffer,int size,int *return_checksum)
{
  int writed,checksum,i;
  writed=write(infile.fp,buffer,size);
  checksum=0;
  for(i=0;i<size;i++) checksum+=buffer[i];
  infile.checksum+=checksum;
  if(return_checksum!=NULL) *return_checksum=checksum;
  return writed;
}

int min(int a, int b)
{
  if ( a < b ) return a;
  return b;
}

void *allocsafe( size_t size )
{
  void *ptr;
  
  ptr = malloc( size );
  if ( ptr == NULL ) {
    printf( "No Memory!\n" );
    exit( 5 );
  }
  return ptr;
}
